import API_CONFIG from "../config/api-config"

interface Options extends RequestInit {
  // headers?: {
  //   body?: object
  //   Origin?: string
  //   "Content-Type"?: string
  // }
  // method?: string
  // mode?: string
  // credentials?: string
  headers?: any
  toggleLoader?: Function
}

export default class Network {
  static async get(url: string, queryParams: object, options: Options) {
    const urlString = Network.stringifyUrl(url, queryParams)
    options = {
      method: "GET",
      mode: "cors",
      credentials: "include",
      ...options,
    }
    return Network.crud(urlString, options)
  }

  static async getStatic(
    url: string,
    queryParams: object | null,
    options: Options | null,
  ) {
    let urlString = Network.stringifyUrl(url, queryParams)
    const environment = process.env.NODE_ENV || "development"
    const host = urlString.startsWith("http")
      ? ""
      : API_CONFIG.STATIC_HOST[environment as keyof object]
    urlString = host ? host + urlString : urlString
    options = {
      method: "GET",
      mode: "cors",
      credentials: "include",
      ...options,
    }
    return Network.crud(urlString, options)
  }

  static async post(url: string, data: object, queryParams: object) {
    const urlString = Network.stringifyUrl(url, queryParams)
    const options: Options = {
      method: "POST",
      body: JSON.stringify(data),
      mode: "cors",
      credentials: "include",
      headers: {
        "Content-Type": "application/json",
      },
    }
    return Network.crud(urlString, options)
  }

  static async put(url: string, data: object, queryParams: object) {
    const urlString = Network.stringifyUrl(url, queryParams)
    const options: Options = {
      method: "PUT",
      body: JSON.stringify(data),
      mode: "cors",
      credentials: "include",
      headers: {
        "Content-Type": "application/json",
      },
    }
    return Network.crud(urlString, options)
  }

  static async delete(url: string, queryParams: object) {
    const urlString = Network.stringifyUrl(url, queryParams)
    const options = {
      method: "DELETE",
      noInject: true,
    }
    return Network.crud(urlString, options)
  }

  static async crud(urlString: string, options: Options) {
    const environment = process.env.NODE_ENV || "development"
    const host = urlString.startsWith("http")
      ? ""
      : API_CONFIG.HOST[environment as keyof object]
    urlString = host ? host + urlString : urlString
    if (!options.headers) {
      options.headers = {}
    }
    options.toggleLoader && options.toggleLoader({ [urlString]: true })
    options.headers["Origin"] =
      window.location.protocol + "//" + window.location.host
    const response = await fetch(urlString, options)
    const { ok, status, headers } = response
    if (ok) {
      if (headers.get("content-type")!.indexOf("application/json") !== -1) {
        const body = await response.json()
        options.toggleLoader && options.toggleLoader({ [urlString]: false })
        return { status, body }
      } else if (
        headers.get("content-type")!.indexOf("application/zip") !== -1 ||
        headers.get("content-type")!.indexOf("application/pdf") !== -1
      ) {
        const body = await response.arrayBuffer()
        options.toggleLoader && options.toggleLoader({ [urlString]: false })
        return { status, body }
      }
      const body = await response.text()
      options.toggleLoader && options.toggleLoader({ [urlString]: false })
      return { status, body }
    }
    const err = await response.json()
    options.toggleLoader && options.toggleLoader({ [urlString]: false })
    console.log(err)
    throw new Error(err.message, err.error)
  }

  static stringifyUrl(url: string, queryParams: any) {
    return url
      ? queryParams
        ? Object.keys(queryParams).reduce(
            (acc, key) => acc.concat(`${key}=${queryParams[key]}`),
            url + "?",
          )
        : url
      : ""
  }
}
